#include "StdAfx.h"
#include "WASGetHanatourCount.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"

CWASGetHanatourCount::CWASGetHanatourCount(CWASActor *pActor) : CWASCommand(pActor)
{
}

CWASGetHanatourCount::~CWASGetHanatourCount(void)
{
}


//////////////////////////////////////
void CWASGetHanatourCount::ExecuteAsync()
{
	m_listHanatourCount.clear();

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	USES_UTF8_CONVERSION;

	WSTRING strURL = m_pActor->GetWASServerAddress();
	strURL += _T("/ne/notice/GetLegacyData.php");

	m_httpFacade.StopRequest();
	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));	
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());	

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}


bool CWASGetHanatourCount::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;
	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return false;
	}	

	USES_UTF8_CONVERSION;

	do
	{
		nfc::IIkXMLNode * pRootNode = pDom->GetdocumentElement()->selectSingleNode("nodes");
		nfc::IIkXMLNodeList *pNodeList;
		nfc::IIkXMLNode * pListNode;
		nfc::IIkXMLAttribute * pAttr;		

		pNodeList = pRootNode->selectNodes("node");

		// Tree Re-Construction

		for(int j = 0; j < pNodeList->Getlength(); j++)
		{
			NE_HANATOUR_COUNT countData;
			memset(&countData, 0x00, sizeof(NE_HANATOUR_COUNT));
			pListNode = pNodeList->Getitem(j);

			pAttr = pListNode->getAttribute("type");
			if(NULL == pAttr) continue;

			if (strcmp(pAttr->Gettext().c_str(), "voc") == 0)
				countData.enumHanatourCountType = hanatourVoc;
			else if (strcmp(pAttr->Gettext().c_str(), "sign") == 0)
				countData.enumHanatourCountType = hanatourSign;
			else if (strcmp(pAttr->Gettext().c_str(), "poll") == 0)
				countData.enumHanatourCountType = hanatourPoll;
			else if (strcmp(pAttr->Gettext().c_str(), "hanain") == 0)
				countData.enumHanatourCountType = hanatourIN;
			else if (strcmp(pAttr->Gettext().c_str(), "hanasam") == 0)
				countData.enumHanatourCountType = hanatourSAM;
			else
				continue;

			pAttr = pListNode->getAttribute("count");
			if(NULL == pAttr) 
				countData.nCount = 0;
			else
				countData.nCount = atoi(pAttr->Gettext().c_str());

			pAttr = pListNode->getAttribute("url");
			if(NULL == pAttr) continue;
			countData.strLinkUrl = U82W(pAttr->Gettext().c_str());

			m_listHanatourCount.push_back(countData);				
		}

	} while(FALSE);

	return true;
}


// override in drived class
void CWASGetHanatourCount::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();
	
	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseGetHanatourCount(m_pActor, GetID(), m_eError, m_listHanatourCount) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetHanatourCount::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();	

	for (; it != observers.end(); ++it)
		if((*it)->OnGTWASActor_ResponseGetHanatourCount(m_pActor, GetID(), m_eError, m_listHanatourCount) == false)
			break;

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}